@@ -19,14 +19,14 @@ import logging
 from time import sleep
 import cysecuretools.execute.provisioning_packet_mxs40v1 as prov_packet
 from cysecuretools.core.target_director import Target
-from cysecuretools.execute.enums import (ProtectionState, EntranceExamStatus,
-                                         ProvisioningStatus, KeyId)
+from cysecuretools.core.enums import (ProtectionState, EntranceExamStatus,
+                                      ProvisioningStatus, KeyId)
 from cysecuretools.execute.entrance_exam.exam_mxs40v1 import \
     EntranceExamMXS40v1
 from cysecuretools.execute.provisioning_lib.cyprov_pem import PemKey
 from cysecuretools.execute.sys_call \
     import (provision_keys_and_policies, read_lifecycle, dap_control,
-            get_prov_details, FB_POLICY_JWT, FB_POLICY_IMG_CERTIFICATE)
+            get_prov_details, FB_POLICY_JWT)
 from cysecuretools.execute.programmer.base import AP
 from cysecuretools.execute.programmer.pyocd_wrapper import ResetType
 from cysecuretools.core.strategy_context.provisioning_strategy_ctx import \
@@ -65,6 +65,11 @@ class ProvisioningMXS40V1(ProvisioningStrategy):
         else:
             ap = 'cm0'
 
+        if 'skip_prompts' in kwargs:
+            skip_prompts = kwargs['skip_prompts']
+        else:
+            skip_prompts = None
+
         if 'probe_id' in kwargs:
             probe_id = kwargs['probe_id']
         else:
@@ -72,7 +77,8 @@ class ProvisioningMXS40V1(ProvisioningStrategy):
 
         prov_packets = self._get_provisioning_packet(target)
         status = _provision_identity(tool, target, entrance_exam,
-                                     prov_packets['prov_identity'])
+                                     prov_packets['prov_identity'],
+                                     skip_prompts)
 
         if status == ProvisioningStatus.OK:
             status = _provision_complete(tool, target,
@@ -143,21 +149,6 @@ class ProvisioningMXS40V1(ProvisioningStrategy):
         """
         erase_flash(tool, target)
 
-    def read_image_certificate(self, tool, target):
-        """
-        Reads image certificate provisioned into device
-        :param tool: Programming/debugging tool
-        :param target: The target object
-        :return: The certificate
-        """
-        passed, data = get_prov_details(tool, target.register_map,
-                                        FB_POLICY_IMG_CERTIFICATE)
-        if passed:
-            return data
-        else:
-            logger.error('Failed to read image certificate from device')
-            return None
-
     @staticmethod
     def _get_provisioning_packet(target):
         packet_dir = target.policy_parser.get_provisioning_packet_dir()
@@ -193,27 +184,9 @@ def read_silicon_data(tool, target: Target):
     :return: Device response
     """
     logger.debug('Read silicon data')
-    logger.debug('Run provision_keys_and_policies syscall')
-    response = None
     tool.reset(ResetType.HW)
-    passed = provision_keys_and_policies(tool, None, target.register_map)
-    if passed:
-        scratch_addr = tool.read32(
-            target.register_map.ENTRANCE_EXAM_SRAM_ADDR + 0x04)
-        resp_size = tool.read32(scratch_addr)
-        resp_addr = tool.read32(scratch_addr + 0x04)
-        logger.debug(f'resp_size = {hex(resp_size)}')
-        logger.debug(f'resp_addr = {hex(resp_addr)}')
-
-        response = ''
-        for i in range(resp_size):
-            hash_byte_chr = chr(tool.read8(resp_addr + i))
-            response += hash_byte_chr
-        response = response.strip()
-        logger.debug(f'ProvisionKeysAndPolicies response = \'{response}\'')
-    else:
-        logger.error('Unexpected ProvisionKeysAndPolicies syscall response')
-
+    passed, response = provision_keys_and_policies(tool, None,
+                                                   target.register_map)
     return response
 
 
@@ -224,8 +197,9 @@ def erase_flash(tool, target):
     logger.info(f'erasing address {hex(addr)}, size {hex(size)} ...')
     ap = tool.get_ap()
     tool.set_ap(AP.CMx)
+    tool.halt()
     tool.erase(addr, size)
-    logger.info('Erasing complete\n')
+    logger.info('Erasing complete')
     tool.set_ap(ap)
     erase_smif(tool, target)
 
@@ -239,7 +213,7 @@ def erase_smif(tool, target):
         for (addr, size) in smif_resources:
             logger.info(f'erasing address {hex(addr)}, size {hex(size)} ...')
             tool.erase(addr, size)
-            logger.info('Erasing complete\n')
+            logger.info('Erasing complete')
         tool.set_ap(ap)
 
 
@@ -258,6 +232,7 @@ def erase_slots(tool, target, slot_type, first_only=False):
         logger.info(f'erasing address {hex(addr)}, size {hex(size)} ...')
         ap = tool.get_ap()
         tool.set_ap(AP.CMx)
+        tool.halt()
         tool.erase(addr, size)
         logger.info('Erasing complete')
         tool.set_ap(ap)
@@ -267,20 +242,22 @@ def erase_slots(tool, target, slot_type, first_only=False):
 
 def _provision_identity(tool, target: Target,
                         entrance_exam: EntranceExamMXS40v1,
-                        prov_identity_jwt) -> ProvisioningStatus:
-    logger.info("CPUSS.PROTECTION state: "
-                "'0': UNKNOWN '1': VIRGIN '2': NORMAL '3': SECURE '4': DEAD")
+                        prov_identity_jwt, skip_prompts) -> ProvisioningStatus:
     lifecycle = read_lifecycle(tool, target.register_map)
-    logger.info(f'{lifecycle}\n')
 
     if lifecycle == ProtectionState.secure:
         status = entrance_exam.execute(tool)
         if status == EntranceExamStatus.FLASH_NOT_EMPTY:
-            answer = input('Erase user firmware running on chip? (y/n): ')
-            if answer.lower() == 'y':
-                erase_flash(tool, target)
+            if skip_prompts:
+                logger.error('Cannot start provisioning. '
+                             'User firmware running on chip detected')
+                return ProvisioningStatus.FAIL
             else:
-                return ProvisioningStatus.TERMINATED
+                answer = input('Erase user firmware running on chip? (y/n): ')
+                if answer.lower() == 'y':
+                    erase_flash(tool, target)
+                else:
+                    return ProvisioningStatus.TERMINATED
         elif status != EntranceExamStatus.OK:
             return ProvisioningStatus.FAIL
     else:
@@ -289,10 +266,12 @@ def _provision_identity(tool, target: Target,
     tool.reset_and_halt()
     sleep(0.2)
 
-    is_exam_pass = provision_keys_and_policies(tool, prov_identity_jwt,
-                                               target.register_map)
+    is_exam_pass, response = provision_keys_and_policies(
+        tool, prov_identity_jwt, target.register_map)
+    _save_device_response(target, response)
+
     if not is_exam_pass:
-        logger.error('Unexpected TransitionToSecure syscall response\n')
+        logger.error('Unexpected TransitionToSecure syscall response')
         return ProvisioningStatus.FAIL
     else:
         return ProvisioningStatus.OK
@@ -309,6 +288,7 @@ def _provision_complete(tool, target: Target, prov_cmd_jwt, bootloader,
         if cm0_open:
             tool.disconnect()
             tool.connect(target.name, probe_id=probe_id, ap='cm0')
+            tool.reset_and_halt(ResetType.HW)
         flash_ops_allowed = cm0_open or ap == 'cm4'
 
     reg_map = target.register_map
@@ -377,6 +357,7 @@ def _provision_complete(tool, target: Target, prov_cmd_jwt, bootloader,
                 current_ap = tool.get_ap()
                 tool.set_ap(AP.CMx)
                 logger.info(f'Programming user application \'{app}\':')
+                tool.halt()
                 tool.program(app)
                 tool.set_ap(current_ap)
             else:
@@ -395,16 +376,19 @@ def _provision_complete(tool, target: Target, prov_cmd_jwt, bootloader,
             logger.error('Bootloader encrypted programming failed')
             return ProvisioningStatus.FAIL
     else:
-        if flash_ops_allowed:
+        if not flash_ops_allowed:
+            logger.warning('Skip programming bootloader, AP cm0 is closed')
+        elif bootloader is None:
+            logger.warning('Skip programming bootloader')
+        else:
             sleep(3)
             current_ap = tool.get_ap()
             tool.set_ap(AP.CMx)
             logger.info(f'Programming bootloader \'{bootloader}\':')
+            tool.halt()
             tool.program(bootloader)
             logger.info('Programming bootloader complete')
             tool.set_ap(current_ap)
-        else:
-            logger.warning('Skip programming bootloader, AP cm0 is closed')
 
     if control_dap_cert:
         tool.set_skip_reset_and_halt(False)
@@ -421,11 +405,13 @@ def _provision_complete(tool, target: Target, prov_cmd_jwt, bootloader,
 
     # Run provisioning syscall
     logger.info('Run provisioning syscall:')
-    is_exam_pass = provision_keys_and_policies(tool, prov_cmd_jwt,
-                                               target.register_map)
+    is_exam_pass, response = provision_keys_and_policies(tool, prov_cmd_jwt,
+                                                         target.register_map)
     if not is_exam_pass:
         return ProvisioningStatus.FAIL
 
+    _save_device_response(target, response)
+
     tool.reset()
 
     if not target.policy_parser.is_sys_ap_enabled():
@@ -477,3 +463,16 @@ def _save_device_public_key(tool, target):
     except Exception as e:
         logger.error('Failed to save device public key')
         logger.error(e)
+
+
+def _save_device_response(target, response):
+    try:
+        packet_dir = target.policy_parser.get_provisioning_packet_dir()
+        filename = os.path.join(packet_dir, prov_packet.DEVICE_RESPONSE_JWT)
+        if response:
+            with open(filename, 'w') as f:
+                f.write(response)
+        logger.info(f'Saved device response to \'{filename}\'')
+    except Exception as e:
+        logger.error('Failed to save device response')
+        logger.error(e)
