@@ -17,13 +17,12 @@ import json
 import logging
 import os
 import sys
-from datetime import datetime
-
 from cryptography.hazmat.primitives import serialization
 
 import cysecuretools.execute.jwt as jwt
 import cysecuretools.execute.keygen as keygen
-from cysecuretools.version import __version__
+from cysecuretools.execute.version_helper import VersionHelper
+from cysecuretools.core.exceptions import ValidationError
 from cysecuretools.core.bootloader_provider import BootloaderProvider
 from cysecuretools.core.certificates.x509 import X509CertificateStrategy
 from cysecuretools.core.logging_formatter import CustomFormatter
@@ -39,16 +38,15 @@ from cysecuretools.core.target_director import TargetDirector
 from cysecuretools.core.project import ProjectInitializer
 from cysecuretools.execute.encrypted_programming.aes_header_strategy \
     import AesHeaderStrategy
-from cysecuretools.execute.enums import (EntranceExamStatus,
-                                         ProvisioningStatus,
-                                         ImageType, KeyType, KeyId)
+from cysecuretools.core.enums import (EntranceExamStatus, ValidationStatus,
+                                      ProvisioningStatus, KeyAlgorithm,
+                                      ImageType, KeyType, KeyId)
 from cysecuretools.execute.image_cert import ImageCertificate
 from cysecuretools.execute.key_reader import get_aes_key
 from cysecuretools.execute.programmer.programmer import ProgrammingTool
 from cysecuretools.execute.signtool import SignTool
-from cysecuretools.targets import print_targets
-from cysecuretools.targets import target_map
-from pyocd.core.exceptions import TransferFaultError, TransferError
+from cysecuretools.targets import print_targets, get_target_builder
+from cysecuretools.core.logging_configurator import LoggingConfigurator
 
 # Initialize logger
 logging.root.setLevel(logging.DEBUG)
@@ -68,18 +66,27 @@ class CySecureTools:
     TOOLS_PATH = os.path.dirname(os.path.realpath(__file__))
     PROGRAMMING_TOOL = 'pyocd'
 
-    def __init__(self, target=None, policy=None, log_file=True):
+    def __init__(self, target=None, policy=None, log_file=True,
+                 skip_prompts=False):
         """
         Creates instance of the class
-        :param target: Device manufacturing part number.
-        :param policy: Provisioning policy file.
+        :param target: Device manufacturing part number
+        :param policy: Provisioning policy file
+        :param log_file: Indicates whether to write log into a file
+        :param skip_prompts: Indicates whether to skip user interactive
+               prompts
         """
         if log_file:
-            add_file_logging()
+            LoggingConfigurator.add_file_logging()
+
+        self.tool = ProgrammingTool.create(self.PROGRAMMING_TOOL)
+
+        self.skip_prompts = skip_prompts
+        if self.skip_prompts:
+            self.tool.wait_for_target = False
 
         self.inited = True
         if not target:
-            print_targets()
             self.inited = False
             return
 
@@ -89,11 +96,10 @@ class CySecureTools:
         self.policy = None
         if policy is not None:
             self.policy = os.path.abspath(policy)
-        else:
-            if ProjectInitializer.is_project():
+        if ProjectInitializer.is_project():
+            if policy is None:
                 self.policy = ProjectInitializer.get_default_policy()
-                self.policy = os.path.abspath(self.policy)
-                cwd = os.getcwd()
+            cwd = os.getcwd()
 
         if self.policy and not os.path.isfile(self.policy):
             raise ValueError(f'Cannot find file "{self.policy}"')
@@ -109,18 +115,21 @@ class CySecureTools:
         self.target_dir = self.target.target_dir
 
         # Validate policy file
-        if not self.policy_validator.validate(skip=['pre_build']):
-            raise RuntimeError('Policy validation failed')
+        validation_state = self.policy_validator.validate(
+            skip=['pre_build', 'dap_disabling'],
+            skip_prompts=self.skip_prompts)
+        if validation_state in [ValidationStatus.ERROR,
+                                ValidationStatus.TERMINATED]:
+            raise ValidationError
 
         self.target.key_reader = self.target.key_reader(self.target)
         self.key_reader = self.target.key_reader
         self.bootloader_provider = BootloaderProvider(self.target)
         self.entr_exam = self.target.entrance_exam(self.target)
         self.project_initializer = self.target.project_initializer(self.target)
-        self.tool = ProgrammingTool.create(self.PROGRAMMING_TOOL)
 
     def create_keys(self, overwrite=None, out=None, kid=None,
-                    user_key_alg='ec'):
+                    user_key_alg=KeyAlgorithm.EC):
         """
         Creates keys specified in policy file for image signing
         :param overwrite: Indicates whether overwrite keys in the
@@ -134,6 +143,16 @@ class CySecureTools:
         :param user_key_alg: User key algorithm
         :return: True if key(s) created successfully, otherwise False.
         """
+        # Define key algorithm
+        if user_key_alg is None:
+            user_key_alg = self.target.key_algorithms[0]
+        else:
+            if user_key_alg not in self.target.key_algorithms:
+                logger.error(f'Invalid key algorithm \'{user_key_alg}\'. '
+                             f'Supported key algorithms for the selected '
+                             f'target: {",".join(self.target.key_algorithms)}')
+                return False
+
         # Find keys that have to be generated
         keys = self.policy_parser.get_keys(out)
 
@@ -169,18 +188,18 @@ class CySecureTools:
                 else:
                     seen.append({pair.key_id, pair.json_key_path})
 
-                if user_key_alg == 'ec':
+                if user_key_alg == KeyAlgorithm.EC:
                     keygen.generate_ecdsa_key(pair.key_id, pair.json_key_path,
                                               pair.pem_key_path)
-                else:
-                    raise ValueError(f'Invalid key algorithm {user_key_alg}.')
+                elif user_key_alg == KeyAlgorithm.RSA:
+                    keygen.generate_rsa_key(pair.pem_key_path)
             else:
                 continue
 
         return True
 
     def sign_image(self, hex_file, image_id=4, image_type=None,
-                   encrypt_key=None):
+                   encrypt_key=None, erased_val=None, boot_record='default'):
         """
         Signs firmware image with the key specified in the policy file.
         :param hex_file: User application hex file.
@@ -188,13 +207,19 @@ class CySecureTools:
         :param image_type: Image type (BOOT or UPGRADE).
         :param encrypt_key: Path to public key file
                for the image encryption
+        :param erased_val: The value that is read back from erased flash
+        :param boot_record: Create CBOR encoded boot record TLV.
+               The sw_type represents the role of the software component
+               (e.g. CoFM for coprocessor firmware). [max. 12 characters]
         :return: Signed (and encrypted if applicable) hex files path.
         """
-        sign_tool = SignTool(self.policy, self.memory_map)
+        sign_tool = SignTool(self.target)
         result = sign_tool.sign_image(hex_file=hex_file,
                                       image_id=image_id,
                                       image_type=image_type,
-                                      encrypt_key=encrypt_key)
+                                      encrypt_key=encrypt_key,
+                                      erased_val=erased_val,
+                                      boot_record=boot_record)
         return result
 
     def create_provisioning_packet(self):
@@ -202,14 +227,16 @@ class CySecureTools:
         Creates JWT packet for provisioning device.
         :return: True if packet created successfully, otherwise False.
         """
-        if not self.policy_validator.validate():
-            logger.error('Policy validation failed')
+        validation_state = self.policy_validator.validate(
+            skip_prompts=self.skip_prompts)
+        if validation_state in [ValidationStatus.ERROR,
+                                ValidationStatus.TERMINATED]:
             return False
 
         filtered_policy = self.policy_filter.filter_policy()
 
         # Get bootloader image certificate
-        image_cert = self.bootloader_provider.get_jwt_path()
+        image_cert = self.bootloader_provider.jwt_path()
         if not os.path.isfile(image_cert):
             logger.error(f'Cannot find bootloader file \'{image_cert}\'')
             return False
@@ -233,29 +260,30 @@ class CySecureTools:
         :param ap: The access port used for provisioning
         :return: Provisioning result. True if success, otherwise False.
         """
-        # Get bootloader program file
-        if not self.policy_validator.validate():
-            logger.error('Policy validation failed')
+        validation_state = self.policy_validator.validate(
+            skip_prompts=self.skip_prompts)
+        if validation_state in [ValidationStatus.ERROR,
+                                ValidationStatus.TERMINATED]:
             return False
 
-        bootloader = self.bootloader_provider.get_hex_path()
+        # Get bootloader program file
+        bootloader = self.bootloader_provider.hex_path()
         if not os.path.isfile(bootloader):
             logger.error(f'Cannot find bootloader file \'{bootloader}\'')
             return False
 
         context = ProvisioningContext(self.target.provisioning_strategy)
 
-        try:
-            if self.tool.connect(self.target_name, probe_id=probe_id, ap=ap):
-                status = context.provision(self.tool, self.target,
-                                           self.entr_exam, bootloader,
-                                           probe_id=probe_id, ap=ap)
-                self.tool.disconnect()
-            else:
-                status = ProvisioningStatus.FAIL
-        except (TransferFaultError, TransferError) as e:
-            logger.error(e)
-            logger.debug(e, exc_info=True)
+        if self.tool.connect(self.target_name, probe_id=probe_id, ap=ap):
+            VersionHelper.log_version(self.tool, self.target)
+            if not VersionHelper.verify_sfb_version(self.tool, self.target):
+                return False
+            status = context.provision(self.tool, self.target,
+                                       self.entr_exam, bootloader,
+                                       probe_id=probe_id, ap=ap,
+                                       skip_prompts=self.skip_prompts)
+            self.tool.disconnect()
+        else:
             status = ProvisioningStatus.FAIL
 
         if status == ProvisioningStatus.FAIL:
@@ -265,7 +293,7 @@ class CySecureTools:
         return True
 
     def re_provision_device(self, probe_id=None, ap='sysap', erase_boot=False,
-                            control_dap_cert=None):
+                            control_dap_cert=None, skip_bootloader=False):
         """
         Executes device re-provisioning
         :param probe_id: Probe serial number.
@@ -273,36 +301,39 @@ class CySecureTools:
         :param erase_boot: Indicates whether erase BOOT slot
         :param control_dap_cert: The certificate that provides the
                access to control DAP
+        :param skip_bootloader: Indicates whether to skip bootloader
+               programming during reprovisioning
         :return: Provisioning result. True if success, otherwise False.
         """
-        # Get bootloader program file
-        if not self.policy_validator.validate():
-            logger.error('Policy validation failed')
+        validation_state = self.policy_validator.validate(
+            skip_prompts=self.skip_prompts)
+        if validation_state in [ValidationStatus.ERROR,
+                                ValidationStatus.TERMINATED]:
             return False
 
-        btldr = self.bootloader_provider.get_hex_path()
-        if not os.path.isfile(btldr):
-            logger.error(f'Cannot find bootloader file \'{btldr}\'')
-            return False
+        # Get bootloader program file
+        btldr = None
+        if not skip_bootloader:
+            btldr = self.bootloader_provider.hex_path()
+            if not os.path.isfile(btldr):
+                logger.error(f'Cannot find bootloader file \'{btldr}\'')
+                return False
 
         context = ProvisioningContext(self.target.provisioning_strategy)
 
-        try:
-            if self.tool.connect(self.target_name, probe_id=probe_id, ap=ap):
-                status = context.re_provision(
-                    self.tool, self.target, btldr, erase_boot=erase_boot,
-                    control_dap_cert=control_dap_cert, ap=ap,
-                    probe_id=probe_id)
-                self.tool.disconnect()
-            else:
-                status = ProvisioningStatus.FAIL
-        except (TransferFaultError, TransferError) as e:
-            logger.error(e)
-            logger.debug(e, exc_info=True)
+        if self.tool.connect(self.target_name, probe_id=probe_id, ap=ap):
+            VersionHelper.log_version(self.tool, self.target)
+            if not VersionHelper.verify_sfb_version(self.tool, self.target):
+                return False
+            status = context.re_provision(
+                self.tool, self.target, btldr, erase_boot=erase_boot,
+                control_dap_cert=control_dap_cert, ap=ap, probe_id=probe_id)
+            self.tool.disconnect()
+        else:
             status = ProvisioningStatus.FAIL
 
         if status == ProvisioningStatus.FAIL:
-            logger.error('Error occurred while provisioning device')
+            logger.error('Error occurred while reprovisioning device')
 
         return status == ProvisioningStatus.OK
 
@@ -328,15 +359,11 @@ class CySecureTools:
 
         if not all_fields_present or not serial or not public_key:
             logger.info('Get default certificate data')
-            try:
-                default = context.default_certificate_data(
-                    self.tool, self.target, self.entr_exam, probe_id)
-                if not default:
-                    logger.error('Failed to get data for the certificate')
-                    return None
-            except (TransferFaultError, TransferError) as e:
-                logger.error(e)
-                logger.debug(e, exc_info=True)
+
+            default = context.default_certificate_data(
+                self.tool, self.target, self.entr_exam, probe_id)
+            if not default:
+                logger.error('Failed to get data for the certificate')
                 return None
 
             for field in expected_fields:
@@ -348,34 +375,41 @@ class CySecureTools:
             if not public_key:
                 kwargs['public_key'] = default['public_key']
 
-        logger.info('Create certificate')
-        return context.create_certificate(cert_name, cert_encoding, **kwargs)
+        logger.info('Start creating certificate')
+        overwrite = True if self.skip_prompts else None
+        return context.create_certificate(cert_name, cert_encoding,
+                                          overwrite=overwrite, **kwargs)
 
-    def entrance_exam(self, probe_id=None, ap='cm4'):
+    def entrance_exam(self, probe_id=None, ap='cm4', erase_flash=False):
         """
         Checks device life-cycle, Flashboot firmware and Flash state.
         :param probe_id: Probe serial number.
         :param ap: The access port used for entrance exam
+        :param erase_flash: Indicates whether to erase flash before the
+               entrance exam
         :return True if the device is ready for provisioning,
                 otherwise False.
         """
         status = False
-        try:
-            if self.tool.connect(self.target_name, probe_id=probe_id, ap=ap):
-                status = self.entr_exam.execute(self.tool)
-                if status == EntranceExamStatus.FLASH_NOT_EMPTY:
+        if self.tool.connect(self.target_name, probe_id=probe_id, ap=ap):
+            VersionHelper.log_version(self.tool, self.target)
+            if not VersionHelper.verify_sfb_version(self.tool, self.target):
+                return False
+            context = ProvisioningContext(self.target.provisioning_strategy)
+            if erase_flash:
+                context.erase_flash(self.tool, self.target)
+            status = self.entr_exam.execute(self.tool)
+            if status == EntranceExamStatus.FLASH_NOT_EMPTY:
+                if self.skip_prompts:
+                    logger.error('Entrance exam failed. '
+                                 'User firmware running on chip detected')
+                    return ProvisioningStatus.FAIL
+                else:
                     answer = input(
                         'Erase user firmware running on chip? (y/n): ')
                     if answer.lower() == 'y':
-                        context = ProvisioningContext(
-                            self.target.provisioning_strategy)
                         context.erase_flash(self.tool, self.target)
-                self.tool.disconnect()
-        except (TransferFaultError, TransferError) as e:
-            logger.error(e)
-            logger.debug(e, exc_info=True)
-            status = EntranceExamStatus.FAIL
-
+            self.tool.disconnect()
         return status == EntranceExamStatus.OK
 
     def flash_map(self, image_id=4, image_type=ImageType.BOOT.name):
@@ -453,11 +487,13 @@ class CySecureTools:
         :param probe_id: Probe serial number.
                Used to read device public key from device.
         """
-        # Get host private key
-        if not self.policy_validator.validate():
-            logger.error('Policy validation failed')
+        validation_state = self.policy_validator.validate(
+            skip=['dap_disabling'], skip_prompts=self.skip_prompts)
+        if validation_state in [ValidationStatus.ERROR,
+                                ValidationStatus.TERMINATED]:
             return False
 
+        # Get host private key
         logger.debug(f'Host key id = {host_key_id}')
         try:
             _, host_key_pem = self.policy_parser.get_private_key(host_key_id)
@@ -468,18 +504,16 @@ class CySecureTools:
         # Get public key
         pub_key_pem = None
         logger.debug(f'Device key id = {dev_key_id}')
-        try:
-            connected = self.tool.connect(self.target_name, probe_id=probe_id,
-                                          blocking=False, ap='sysap')
-            if connected:
-                logger.info('Read device public key from device')
-                pub_key_pem = self.key_reader.read_public_key(
-                    self.tool, dev_key_id, 'pem')
-                self.tool.disconnect()
-        except (TransferFaultError, TransferError) as e:
-            logger.error(e)
-            logger.debug(e, exc_info=True)
-            return False
+
+        connected = self.tool.connect(self.target_name, probe_id=probe_id,
+                                      blocking=False, ap='sysap')
+        if connected:
+            VersionHelper.log_version(self.tool, self.target)
+            VersionHelper.verify_sfb_version(self.tool, self.target)
+            logger.info('Read device public key from device')
+            pub_key_pem = self.key_reader.read_public_key(
+                self.tool, dev_key_id, 'pem')
+            self.tool.disconnect()
 
         if not connected or not pub_key_pem:
             logger.info(f'Read public key {dev_key_id} from file')
@@ -509,15 +543,13 @@ class CySecureTools:
         :param probe_id: Probe serial number.
         :return: True if the image programmed successfully, otherwise False.
         """
+        result = False
         context = EncryptedProgrammingContext(AesHeaderStrategy)
-        try:
-            self.tool.connect(self.target_name, probe_id=probe_id, ap='sysap')
+        if self.tool.connect(self.target_name, probe_id=probe_id, ap='sysap'):
+            VersionHelper.log_version(self.tool, self.target)
+            VersionHelper.verify_sfb_version(self.tool, self.target)
             result = context.program(self.tool, self.target, encrypted_image)
             self.tool.disconnect()
-        except (TransferFaultError, TransferError) as e:
-            logger.error(e)
-            logger.debug(e, exc_info=True)
-            result = False
         return result
 
     def read_public_key(self, key_id, key_fmt, out_file=None, probe_id=None):
@@ -529,29 +561,43 @@ class CySecureTools:
         :param probe_id: Probe serial number
         :return: Key if it read successfully, otherwise None
         """
-        try:
-            self.tool.connect(self.target_name, probe_id=probe_id, ap='sysap')
-            key = self.key_reader.read_public_key(self.tool, key_id, key_fmt)
-            if out_file:
-                out_file = os.path.abspath(out_file)
-                with open(out_file, 'w') as fp:
-                    if key_fmt == 'jwk':
-                        json.dump(key, fp, indent=4)
-                    elif key_fmt == 'pem':
-                        fp.write(key.decode("utf-8"))
-                    else:
-                        fp.write(str(key))
-                logger.info(f'Key saved: {out_file}')
-            self.tool.disconnect()
-        except (ValueError, FileNotFoundError) as e:
-            logger.error(e)
-            key = None
-        except (TransferFaultError, TransferError) as e:
-            logger.error(e)
-            logger.debug(e, exc_info=True)
-            key = None
+        key = None
+        if self.tool.connect(self.target_name, probe_id=probe_id, ap='sysap'):
+            VersionHelper.log_version(self.tool, self.target)
+            VersionHelper.verify_sfb_version(self.tool, self.target)
+            try:
+                key = self.key_reader.read_public_key(self.tool, key_id, key_fmt)
+                if out_file:
+                    out_file = os.path.abspath(out_file)
+                    with open(out_file, 'w') as fp:
+                        if key_fmt == 'jwk':
+                            json.dump(key, fp, indent=4)
+                        elif key_fmt == 'pem':
+                            fp.write(key.decode("utf-8"))
+                        else:
+                            fp.write(str(key))
+                    logger.info(f'Key saved: {out_file}')
+                self.tool.disconnect()
+            except (ValueError, FileNotFoundError) as e:
+                logger.error(e)
         return key
 
+    def read_die_id(self, probe_id=None, ap='sysap'):
+        """
+        Reads die ID
+        :param probe_id: Probe serial number
+        :param ap: The access port used to read the data
+        :return: Die ID if success, otherwise None
+        """
+        die_id = None
+        if self.tool.connect(self.target_name, probe_id=probe_id, ap=ap):
+            VersionHelper.log_version(self.tool, self.target)
+            VersionHelper.verify_sfb_version(self.tool, self.target)
+            reader = self.target.silicon_data_reader(self.target)
+            die_id = reader.read_die_id(self.tool)
+            self.tool.disconnect()
+        return die_id
+
     def sign_json(self, json_file, priv_key_id, output_file):
         """
         Signs JSON file with the private key
@@ -564,9 +610,11 @@ class CySecureTools:
         :return: The JWT
         """
         logger.info(f'Signing file {os.path.abspath(json_file)}')
-        if not self.policy_validator.validate():
-            logger.error('Policy validation failed')
-            return None
+        validation_state = self.policy_validator.validate(
+            skip=['dap_disabling'], skip_prompts=self.skip_prompts)
+        if validation_state in [ValidationStatus.ERROR,
+                                ValidationStatus.TERMINATED]:
+            return False
 
         logger.debug(f'Private key id = {priv_key_id}')
         try:
@@ -591,110 +639,53 @@ class CySecureTools:
         :param ap: The access port used for to read CyBootloader and
                Secure Flash Boot version from device
         """
-        context = ProvisioningContext(self.target.provisioning_strategy)
-        sfb_version = 'unknown'
-        connected = False
-        cert = None
-        try:
-            connected = self.tool.connect(self.target_name, probe_id=probe_id,
-                                          blocking=False, ap=ap)
-            if connected:
-                sfb_version = self.entr_exam.read_sfb_version(self.tool)
-                cert = context.read_image_certificate(self.tool, self.target)
-                self.tool.disconnect()
-        except (TransferFaultError, TransferError) as e:
-            logger.error(e)
-            logger.debug(e, exc_info=True)
-            cert = None
+        sfb_ver = 'unknown'
+        connected = self.tool.connect(self.target_name, probe_id=probe_id,
+                                      blocking=False, ap=ap)
+        if connected:
+            sfb_ver = VersionHelper.sfb_version(self.tool, self.target)
+            bootloader_ver = VersionHelper.device_bootloader_version(self.tool, self.target)
+            VersionHelper.verify_sfb_version(self.tool, self.target)
+            self.tool.disconnect()
 
-        print_version([self.target_name])
+        VersionHelper.print_version([self.target_name])
         if connected:
-            if cert:
-                bootloader_version = ImageCertificate.get_version(cert)
-            else:
-                bootloader_version = 'unknown'
             print('Device:')
-            print(f'\tCyBootloader: {bootloader_version}')
-            print(f'\tSecure Flash Boot: {sfb_version}')
+            print(f'\tCyBootloader: {bootloader_ver}')
+            print(f'\tSecure Flash Boot: {sfb_ver}')
 
     def init(self):
         """
         Initializes new project
         """
         cwd = os.getcwd()
-        self.project_initializer.init(cwd)
+        overwrite = True if self.skip_prompts else None
+        self.project_initializer.init(cwd, overwrite)
 
-    @staticmethod
-    def get_target_builder(director, target_name):
-        try:
-            director.builder = target_map[target_name]['class']()
-            return director.builder
-        except KeyError:
-            raise ValueError(f'Unknown target "{target_name}"')
+    def get_probe_list(self):
+        """
+        Gets list of all connected probes
+        """
+        return self.tool.get_probe_list()
+
+    def get_device_info(self, probe_id=None, ap='sysap'):
+        """
+        Gets device information - silicon ID, silicon revision, family ID
+        """
+        connected = self.tool.connect(self.target_name, probe_id=probe_id,
+                                      ap=ap)
+        dev_info = None
+        if connected:
+            dev_info = self.entr_exam.read_device_info(self.tool)
+            self.tool.disconnect()
+        return dev_info
 
     def _get_target(self, target_name, policy, cwd):
         director = TargetDirector()
-        self.target_builder = self.get_target_builder(director, target_name)
+        self.target_builder = get_target_builder(director, target_name)
         return director.get_target(policy, target_name, cwd)
 
     @staticmethod
     def device_list():
         print_targets()
         return True
-
-
-def print_version(targets):
-    """
-    Prints the package version and CyBootloader version bundled with
-    the package for the specified targets
-    :param targets: The list of targets
-    """
-    versions = []
-    for target_name in targets:
-        director = TargetDirector()
-        CySecureTools.get_target_builder(director, target_name)
-        target = director.get_target(None, target_name, None)
-        policy_path = target.policy
-        tools = CySecureTools(target.name, policy_path, log_file=False)
-        jwt_filename = tools.bootloader_provider.get_jwt_path(mode='release')
-        if not os.path.isfile(jwt_filename):
-            logger.error(f'File {jwt_filename} not found')
-            continue
-        version = ImageCertificate.get_version(jwt_filename)
-        if len(targets) == 1:
-            versions.append(f'{version}')
-        else:
-            target_name = target_map[target.name]["display_name"]
-            versions.append(f'\t\t{target_name}: {version}')
-
-    print('\nPackage:')
-    print(f'\tCySecureTools: {__version__}')
-    end_str = '' if len(targets) == 1 or not versions else '\n'
-    print('\tCyBootloader: ', end=end_str)
-    if versions:
-        for item in versions:
-            print(item)
-    else:
-        print('unknown')
-
-
-def set_logger_level(level):
-    root_logger = logging.getLogger()
-    root_logger.setLevel(level)
-    for handler in root_logger.handlers:
-        if isinstance(handler, type(logging.StreamHandler())):
-            handler.setLevel(level)
-
-
-def add_file_logging():
-    if ProjectInitializer.is_project():
-        cwd = os.getcwd()
-    else:
-        cwd = os.path.dirname(os.path.abspath(__file__))
-    log_filename = datetime.now().strftime(
-        os.path.join(cwd, 'logs/cysecuretools_%Y-%m-%d_%H-%M-%S.log'))
-    os.makedirs(os.path.dirname(log_filename), exist_ok=True)
-    file_handler = logging.FileHandler(log_filename, mode='w+')
-    file_handler.setLevel(logging.DEBUG)
-    file_handler.setFormatter(fmt)
-    logger.root.addHandler(file_handler)
