@@ -17,8 +17,10 @@ import jsonschema
 import logging
 import json
 import os.path
+from math import ceil
 from cysecuretools.targets.common.policy_parser import ImageType
 from cysecuretools.core import PolicyValidatorBase
+from cysecuretools.core.enums import ValidationStatus
 from collections import namedtuple
 from cysecuretools.execute.key_reader import load_key
 
@@ -42,12 +44,17 @@ class PolicyValidator(PolicyValidatorBase):
         self.memory_map = memory_map
         self.policy_dir = self.parser.policy_dir
         self.stage = self.get_policy_stage()
+        self.is_smif = self._is_smif()
+        self.is_multi_image = self._is_multi_image()
 
-    def validate(self, skip=None):
+    def validate(self, skip=None, skip_prompts=False):
         """
         Validation of policy.json.
-        :return True if validation succeeds, otherwise False.
+        :param skip: Validators to skip (e.g. pre_build, dap_disabling)
+        :param skip_prompts: Indicates whether to skip interactive prompts
+        :return Validation status
         """
+        status = ValidationStatus.OK
         skip_list = skip if skip else []
         # First stage validation
         with open(POLICY_SCHEMA) as f:
@@ -56,103 +63,175 @@ class PolicyValidator(PolicyValidatorBase):
 
         try:
             jsonschema.validate(self.parser.json, json_schema)
+            logger.debug('First stage validation success')
         except (jsonschema.exceptions.ValidationError,
                 jsonschema.exceptions.SchemaError) as e:
             logger.error('Validation against schema failed')
             logger.error(e)
-            return False
-        logger.debug('First stage validation success')
+            status = ValidationStatus.ERROR
 
         # Second stage validation
-        is_multi_image = self.is_multi_image()
-        logger.debug('Validating firmware slots overlapping')
-        result = self.validate_address_overlap(slot_overlaps=is_multi_image)
-        if not result:
-            return result
-
-        for slot in self.parser.json['boot_upgrade']['firmware'][1:]:
-            boot_auth = slot['boot_auth'][0]
-            boot_keys = slot['boot_keys'][0]
-            logger.debug('Validating boot_auth id matches kid in JSON key file')
-            result = self.key_id_validation(boot_auth, boot_keys)
+        if self._continue_validation(status):
+            is_multi_image = self._is_multi_image()
+            logger.debug('Validating firmware slots overlapping')
+            result = self.validate_address_overlap(is_multi_image)
             if not result:
-                return result
+                status = ValidationStatus.ERROR
 
-        logger.debug('Validating there is no different JWKs with the same key ID')
-        result = self.key_name_validation()
-        if not result:
-            return result
+        if self._continue_validation(status):
+            for slot in self.parser.json['boot_upgrade']['firmware'][1:]:
+                logger.debug('Validating boot_auth id matches kid in JSON '
+                             'key file')
+                result = self.validate_boot_keys(slot)
+                result &= self.validate_key_file_kid(slot)
+                if not result:
+                    status = ValidationStatus.ERROR
 
-        if self.stage == 'multi':
-            logger.debug('Validating multi-image IDs')
-            result = self.validate_multi_image_id()
+        if self._continue_validation(status):
+            logger.debug('Validating there is no different JWKs with the '
+                         'same key ID')
+            result = self.key_name_validation()
             if not result:
-                return result
+                status = ValidationStatus.ERROR
 
-            logger.debug('Validating multi-image smif_id')
-            result = self.validate_multi_image_smif_id()
+        if self._continue_validation(status):
+            if self.stage == 'multi':
+                logger.debug('Validating multi-image IDs')
+                result = self.validate_multi_image_id()
+                if not result:
+                    status = ValidationStatus.ERROR
+
+        if self._continue_validation(status):
+            if self.stage == 'multi':
+                logger.debug('Validating multi-image smif_id')
+                result = self.validate_multi_image_smif_id()
+                if not result:
+                    status = ValidationStatus.ERROR
+
+        if self._continue_validation(status):
+            if self.stage != 'multi':
+                logger.debug('Validating whether image ID corresponds to '
+                             'CyBootloader launch ID')
+                result = self.image_launch_validation()
+                if not result:
+                    status = ValidationStatus.ERROR
+
+        if self._continue_validation(status):
+            logger.debug('Validating policy for BOOT sections, '
+                         'encryption and SMIF')
+            result = self.check_slots()
             if not result:
-                return result
+                status = ValidationStatus.ERROR
 
-        if self.stage != 'multi':
-            logger.debug('Validating whether image ID corresponds to '
-                         'CyBootloader launch ID')
-            result = self.image_launch_validation()
+        if self._continue_validation(status):
+            logger.debug('Validating monotonic counter')
+            result = self.validate_monotonic_counter()
             if not result:
-                return result
+                status = ValidationStatus.ERROR
 
-        logger.debug('Validating policy for BOOT sections, encryption and SMIF')
-        result = self.check_slots()
-        if not result:
-            return result
+        if self._continue_validation(status):
+            logger.debug('Validating CyBootloader paths')
+            result = self.validate_cybootloader_paths()
+            if not result:
+                status = ValidationStatus.ERROR
 
-        logger.debug('Validating CyBootloader paths')
-        result = self.validate_cybootloader_paths()
-        if not result:
-            return result
+        if self._continue_validation(status):
+            logger.debug('Validate whether slots address value is aligned '
+                         'with the SMPU address limits')
+            result = self.validate_slot_address_alignment()
+            if not result:
+                status = ValidationStatus.ERROR
 
-        logger.debug('Check aligning to Memory map')
-        result = self.memory_map_align()
-        if not result:
-            return result
+        if self._continue_validation(status):
+            logger.debug('Check aligning to Memory map')
+            result = self.memory_map_align()
+            if not result:
+                status = ValidationStatus.ERROR
 
-        if 'pre_build' not in skip_list:
-            logger.debug('Checking integrity of pre-build section')
-            result = self.validate_prebuild_section()
+        if self._continue_validation(status):
+            if 'pre_build' not in skip_list:
+                logger.debug('Checking integrity of pre-build section')
+                result = self.validate_prebuild_section()
+                if not result:
+                    status = ValidationStatus.ERROR
+
+        if self._continue_validation(status):
+            if 'dap_disabling' not in skip_list:
+                result = self.validate_dap_disabling(skip_prompts)
+                if result != ValidationStatus.OK:
+                    status = result
+
+        if self._continue_validation(status):
+            if self.parser.upgrade_mode() == 'swap':
+                result = self.validate_status_partition_size()
+                if not result:
+                    status = ValidationStatus.ERROR
+
+        if status == ValidationStatus.ERROR:
+            logger.error('Policy validation finished with error')
+        elif status == ValidationStatus.WARNING:
+            logger.warning('Policy validation finished with warnings')
+        elif status == ValidationStatus.TERMINATED:
+            logger.info('Terminated by user')
+        else:
+            logger.debug('Second stage validation success')
+
+        return status
+
+    @staticmethod
+    def _continue_validation(status):
+        return status not in [ValidationStatus.ERROR,
+                              ValidationStatus.TERMINATED]
+
+    @staticmethod
+    def validate_boot_keys(slot):
+        """ Validates boot_keys section against boot_auth section """
+        result = True
+        for a in slot['boot_auth']:
+            key_found = False
+            for k in slot['boot_keys']:
+                if k['kid'] == a:
+                    key_found = True
+                    break
+            result &= key_found
+            if not result:
+                logger.error(f'Key with ID {a} not found in the \'boot_keys\' '
+                             f'section (image ID {slot["id"]})')
+                break
 
-        logger.debug('Second stage validation success')
         return result
 
-    def key_id_validation(self, auth, keys):
-        """
-        Validates keys ID in policy.
-        :param auth: Auth ID from policy.
-        :param keys: Key ID from policy.
-        :return True if validation succeeds, otherwise False.
-        """
-        key_file = os.path.join(self.policy_dir, keys['key'])
-        if os.path.exists(key_file):
-            priv, pub = load_key(key_file)
+    def validate_key_file_kid(self, slot):
+        """ Validates kid field of the key files against boot_auth section """
+        result = True
+        for a in slot['boot_auth']:
+            key_found = False
+            for k in slot['boot_keys']:
+                if k['kid'] == a:
+                    key_file = os.path.join(self.policy_dir, k['key'])
+                    if os.path.exists(key_file):
+                        private, public = load_key(key_file)
+                        if private:
+                            file_kid = int(private['kid'])
+                        elif public:
+                            file_kid = int(public['kid'])
+                        else:
+                            file_kid = None
 
-            if priv:
-                key_kid = priv['kid']
-            elif pub:
-                key_kid = pub['kid']
-            else:
-                key_kid = '-1'
+                        if file_kid == a:
+                            key_found = True
+                            break
+                    else:
+                        key_found = True  # it is ok if file does not exist
+                        logger.debug(f'Key file "{key_file}" does not exist')
+            result &= key_found
+            if not result:
+                logger.error(f"Key ID {a}, specified in the 'boot_auth' "
+                             f"section does not match the ID in the 'kid' "
+                             f"field of JWK (image ID {slot['id']})")
+                break
 
-            key_kid = int(key_kid)
-            boot_key_kid = int(keys['kid'])
-
-            if not key_kid == auth:
-                logger.error(f'ID:"{auth}" NOT equals to kid:"{key_kid}" in JSON key file')
-                return False
-            if not boot_key_kid == auth:
-                logger.error(f'ID:"{auth}" NOT equals to kid:"{boot_key_kid}" in JSON key file')
-                return False
-        else:
-            logger.debug(f'Key file "{key_file}" does not exist')
-        return True
+        return result
 
     def key_name_validation(self):
         """
@@ -345,42 +424,61 @@ class PolicyValidator(PolicyValidatorBase):
         :return: True if there are no overlaps, otherwise False.
         """
         AddressSize = namedtuple("AddressSize", "address size")
-        all_addresses = []
+        all_slots_addr = []
+        res_area = []
         for slot in self.parser.json['boot_upgrade']['firmware']:
-            # Create list of used addresses
-            slot_addresses = []
-            for res in slot['resources']:
-                if res['type'] in ['BOOT', 'UPGRADE']:
-                    slot_addresses.append(AddressSize(res['address'],
-                                                      res['size']))
+            app_area = []
+            for item in slot['resources']:
+                if item['type'] in ['BOOT', 'UPGRADE']:
+                    app_area.append(AddressSize(item['address'], item['size']))
+                if item['type'] in ['FLASH_PC1_SPM', 'STATUS_PARTITION']:
+                    res_area.append(AddressSize(item['address'], item['size']))
+
             # Validate overlaps in range of the slot
-            if slot_addresses:
-                result = self.check_overlaps(slot_addresses)
+            if app_area:
+                result = self.check_overlaps(app_area)
                 if not result:
                     return result
-                all_addresses.extend(slot_addresses)
+                all_slots_addr.extend(app_area)
+
+            # Validate overlaps in range of the resources
+            if res_area:
+                result = self.check_overlaps(res_area)
+                if not result:
+                    return result
+
+        # Validate overlaps between app and resources
+        result = self.check_overlaps(all_slots_addr, res_area)
+        if not result:
+            return result
 
         # Validate overlaps between the slots
         if slot_overlaps:
-            return self.check_overlaps(all_addresses)
+            return self.check_overlaps(all_slots_addr)
 
         return True
 
     @staticmethod
-    def check_overlaps(addr_list):
+    def check_overlaps(addr_list1, addr_list2=None):
         """
-        Checks whether addresses in the specified list overlap
+        Checks whether addresses in the specified list(s) overlap.
+        If single list specified, check overlaps in the range of the list.
+        If two lists specified, check overlaps between the lists.
         :return: True if address intersection detected, otherwise False
         """
-        for i in range(len(addr_list)):
-            for k in range(len(addr_list)):
-                if i != k:
-                    x = range(addr_list[i].address,
-                              addr_list[i].address + addr_list[i].size)
-                    y = range(addr_list[k].address,
-                              addr_list[k].address + addr_list[k].size)
-                    xs = set(x)
-                    if len(xs.intersection(y)) > 0:
+        single_list = addr_list2 is None
+        if single_list:
+            addr_list2 = addr_list1
+
+        for i in range(len(addr_list1)):
+            for k in range(len(addr_list2)):
+                if not single_list or (single_list and i != k):
+                    x = range(addr_list1[i].address,
+                              addr_list1[i].address + addr_list1[i].size)
+                    y = range(addr_list2[k].address,
+                              addr_list2[k].address + addr_list2[k].size)
+                    xy = range(max(x.start, y.start), min(x.stop, y.stop))
+                    if len(xy) > 0:
                         logger.error(f'Address range '
                                      f'\'{hex(x.start)}-{hex(x.stop)}\' '
                                      f'overlaps address range '
@@ -418,7 +516,8 @@ class PolicyValidator(PolicyValidatorBase):
         for item in self.parser.json["boot_upgrade"]["firmware"]:
             for res in item["resources"]:
                 if res["type"].startswith("FLASH") or \
-                   res["type"].startswith("BOOT"):
+                   res["type"].startswith("BOOT") or \
+                   res["type"].startswith("STATUS_PARTITION"):
                     flash_res.append([res["address"], res["address"] + res["size"]])
                 if item["upgrade"] and res["type"].startswith("UPGRADE"):
                     if item["smif_id"] == 0:
@@ -431,7 +530,7 @@ class PolicyValidator(PolicyValidatorBase):
         flash_end = flash_start + self.memory_map.FLASH_SIZE
         for item in flash_res:
             if flash_start > item[0] or flash_end < item[0] or flash_end < item[1]:
-                logger.error(f'Address range \'{hex(item[0])}-{hex(item[1])}\' is not in FLASH area (\'{hex(flash_start)}-{hex(flash_end)}\').')
+                logger.error(f'Address range \'{hex(item[0])}-{hex(item[1])}\' is out of available FLASH area (\'{hex(flash_start)}-{hex(flash_end)}\').')
                 return False
 
         # test smif
@@ -547,8 +646,188 @@ class PolicyValidator(PolicyValidatorBase):
 
         return result
 
-    def is_multi_image(self):
+    def _is_multi_image(self):
         for slot in self.parser.json['boot_upgrade']['firmware']:
             if 'multi_image' in slot:
                 return True
         return False
+
+    def validate_slot_address_alignment(self):
+        """
+        Validates whether slots address is aligned with the SMPU
+        address limits
+        """
+        for slot in self.parser.json['boot_upgrade']['firmware']:
+            for res in slot['resources']:
+                if res['type'] in ['BOOT', 'UPGRADE']:
+                    address = int(res['address'])
+                    if address % 1024 != 0:
+                        logger.error(f'{res["type"]} slot address '
+                                     f'({hex(address)}) is misaligned '
+                                     f'with the SMPU address limits for the '
+                                     f'CYB06XXX part')
+                        return False
+        return True
+
+    def validate_dap_disabling(self, skip_prompts=False):
+        """
+        Validates DAP closure and warns a user about the result
+        """
+        cm0_closed = self.parser.is_ap_disabled('m0p')
+        cm4_closed = self.parser.is_ap_disabled('m4')
+        sysap_closed = self.parser.is_ap_disabled('system')
+        result = ValidationStatus.OK
+        if cm0_closed and cm4_closed:
+            if skip_prompts:
+                logger.warning('The policy will close out SWD ports on the '
+                               'chip, preventing any reprogramming of '
+                               'application image via SWD')
+                result = ValidationStatus.WARNING
+            else:
+                answer = input('The policy will close out SWD ports on the '
+                               'chip, preventing any reprogramming of '
+                               'application image via SWD. Please ensure you '
+                               'have a valid image programmed on the chip '
+                               'before provisioning the chip. '
+                               'Continue? (y/n): ')
+                if answer.lower() == 'y':
+                    result = ValidationStatus.OK
+                else:
+                    return ValidationStatus.TERMINATED
+
+        if sysap_closed:
+            if skip_prompts:
+                logger.warning('The policy will close out System DAP on the '
+                               'chip, preventing further reprovisioning')
+                result = ValidationStatus.WARNING
+            else:
+                answer = input('The policy will close out System DAP on the '
+                               'chip, preventing further reprovisioning. '
+                               'Continue? (y/n): ')
+                if answer.lower() != 'y':
+                    return ValidationStatus.TERMINATED
+        return result
+
+    def validate_monotonic_counter(self):
+        """
+        Validates 'monotonic' field in the policy
+        """
+        result = True
+        cm0_monotonic = None
+        cm4_monotonic = None
+        for slot in self.parser.json['boot_upgrade']['firmware']:
+            if slot['id'] != 0:
+                if self._is_cm0_slot(slot['id']):
+                    r = range(0, 16)
+                    cm0_monotonic = slot['monotonic']
+                    if cm0_monotonic not in r:
+                        logger.error(f'CM0 application monotonic counter must '
+                                     f'be in range {r.start}-{r.stop-1}')
+                        result &= False
+                if self._is_cm4_slot(slot['id']):
+                    r = range(8, 16)
+                    cm4_monotonic = slot['monotonic']
+                    if cm4_monotonic not in r:
+                        logger.error(f'CM4 application monotonic counter must '
+                                     f'be in range {r.start}-{r.stop-1}')
+                        result &= False
+        if cm0_monotonic and cm4_monotonic and cm0_monotonic == cm4_monotonic:
+            logger.error('\'monotonic\' field must be different for different '
+                         'applications in multi-image case')
+            result &= False
+        return result
+
+    def validate_status_partition_size(self):
+        result = True
+
+        status_partition = self.parser.status_partition()
+        if not status_partition:
+            raise ValueError('Status partition not found in the specified '
+                             'policy file')
+
+        if self.is_multi_image:
+            d_cm0 = self._calc_status_partition_size('cm0')
+            d_cm4 = self._calc_status_partition_size('cm4')
+            d = d_cm0 + d_cm4
+        else:
+            d = self._calc_status_partition_size()
+
+        if status_partition.size < d:
+            logger.error(f'SWAP status partition is too small. The minimum '
+                         f'sufficient size is {d} bytes')
+            result = False
+
+        return result
+
+    def _calc_status_partition_size(self, slot_name=None):
+        """
+        Calculates minimum sufficient status partition size
+        for the specified slot
+        :param slot_name: cm0 or cm4. If None, the maximum image size
+        between both slots will be used
+        :return: Status partition size in bytes
+        """
+        int_mem_sector_size = 512  # internal memory sector size
+        ext_mem_sector_size = 256 * 1024  # external memory sector size
+        trailer = int_mem_sector_size  # trailer is 64B, one slice is enough
+        duplicates = 2  # status partition duplicates
+        boot_max_size = self._max_image_size('BOOT', slot_name)
+        if self.is_smif:
+            sector_size = ext_mem_sector_size
+        else:
+            sector_size = int_mem_sector_size
+        boot_sectors = boot_max_size / sector_size
+        sector_count = ceil(boot_sectors / sector_size)
+        dx = int_mem_sector_size * sector_count + trailer
+        d = dx * duplicates
+        d = d * 2  # add UPGRADE slot
+        return d
+
+    @staticmethod
+    def _is_bootloader_slot(image_id):
+        return image_id == 0
+
+    @staticmethod
+    def _is_cm0_slot(image_id):
+        return image_id in [1, 2, 3]
+
+    @staticmethod
+    def _is_cm4_slot(image_id):
+        return image_id in [4, 16]
+
+    @staticmethod
+    def _is_app_slot(image_id):
+        return PolicyValidator._is_cm0_slot(image_id) or \
+               PolicyValidator._is_cm4_slot(image_id)
+
+    def _is_smif(self):
+        for slot in self.parser.json['boot_upgrade']['firmware']:
+            if 'smif_id' in slot and slot['smif_id'] > 0:
+                return True
+        return False
+
+    def _max_image_size(self, image_type, slot_name=None):
+        """
+        Gets the maximum size between the images of the
+        specified type and slot
+        """
+        img_max_size = 0
+        max_size_list = dict()
+        for slot in self.parser.json['boot_upgrade']['firmware']:
+            for item in slot['resources']:
+                if item['type'] == image_type:
+                    if item['size'] > img_max_size:
+                        if self._is_cm0_slot(slot['id']):
+                            max_size_list['cm0'] = item['size']
+                        if self._is_cm4_slot(slot['id']):
+                            max_size_list['cm4'] = item['size']
+        if 'cm0' == slot_name:
+            img_max_size = max_size_list['cm0']
+        elif 'cm4' == slot_name:
+            img_max_size = max_size_list['cm4']
+        else:
+            try:
+                img_max_size = max(max_size_list['cm0'], max_size_list['cm4'])
+            except KeyError:
+                img_max_size = list(max_size_list.values())[0]
+        return img_max_size
