@@ -15,6 +15,7 @@ limitations under the License.
 """
 import os
 from cysecuretools.core import TargetBuilder
+from cysecuretools.core.enums import KeyAlgorithm
 from cysecuretools.targets.cyb06xx5.maps.memory_map import MemoryMap_cyb06xx5
 from cysecuretools.targets.cyb06xx5.maps.register_map import \
     RegisterMap_cyb06xx5
@@ -30,12 +31,14 @@ from cysecuretools.execute.project_init_mxs40v1 import \
     ProjectInitializerMXS40V1
 from cysecuretools.execute.voltage_tool_mxs40v1 import VoltageToolMXS40v1
 from cysecuretools.execute.key_reader import KeyReaderMXS40V1
+from cysecuretools.execute.silicon_data_reader_mxs40v1 import \
+    SiliconDataReaderMXS40v1
 
 
 class CYB06xx5_Builder(TargetBuilder):
     def get_default_policy(self):
         return os.path.join(self.target_dir, 'policy',
-                            'policy_multi_CM0_CM4.json')
+                            'policy_single_CM0_CM4.json')
 
     def get_memory_map(self):
         memory_map = MemoryMap_cyb06xx5()
@@ -75,3 +78,9 @@ class CYB06xx5_Builder(TargetBuilder):
 
     def get_project_initializer(self):
         return ProjectInitializerMXS40V1
+
+    def get_silicon_data_reader(self):
+        return SiliconDataReaderMXS40v1
+
+    def get_key_algorithms(self):
+        return [KeyAlgorithm.EC]
